@@ -44,11 +44,21 @@ is.synthACS <- function(x) {
   inherits(x, "synthACS")
 }
 
+#' @title Check smsm_set class
+#' @description Function that checks if the target object is a \code{smsm_set} object.
+#' @param x any R object.
+#' @return Returns \code{TRUE} if its argument has class "macroACS" among its classes and
+#' \code{FALSE} otherwise.
+#' @export
+is.smsm_set <- function(x) {
+  inherits(x, "smsm_set")
+}
+
 ##---------------------------------------------------------
 ## Generics for class macroACS
 ##---------------------------------------------------------
 # This set of generic fetches "estimate" or "standard error" data for each of the datasets
-# contained in the 'macroACS' class. One method is specified for each dataset
+# contained in the 'macroACS' class. 
 ##---------------------------------------------------------
 
 ####  helper functions ####
@@ -70,286 +80,172 @@ validate_get_inputs <- function(acs, geography, dataset= c("estimate", "st.err")
   if (!is.character(geography)) {
     stop("geography must be specified as a character vector.")
   } else {
-    if (any(nchar(geography) < 4) & geography != "*") 
+    if (geography != "*" && any(nchar(geography) < 4)) 
       stop("Please specify at least 4 characters for geography.")
   }
   # if no error, okay
 }
 
-# 3. save duplication on getting datasets
-fetch_data <- function(acs, dataset= c("estimate", "st.err"), 
-                       choice= c("age_by_sex", "pop_by_race", "marital_status", "edu", "nativity", 
-                            "by_inc_12mo", "geo_mob_edu", "ind_inc", "emp_status", "pov_status1")) {
+#' @title Get Aggregate Data Specified Geography
+#' @description Gets aggregate, macro, data, either estimate or standard error, for a specified geography
+#' and specified dataset.
+#' @param acs An object of class \code{"macroACS"}.
+#' @param geography A character vector allowing string matching via \code{\link[base]{grep}} to 
+#' a set of specified geographies. All values may be specified by \code{"*"}.
+#' @param dataset Either \code{"estimate"} or \code{"st.err"}. Do you want data on estimated 
+#' population counts or estimated standard errors?
+#' @param choice A character vector specifying the name of one of the datasets in \code{acs} 
+#' @export
+fetch_data <- function(acs, geography, dataset= c("estimate", "st.err"), 
+                       choice= NULL) {
+  UseMethod("fetch_data", acs) 
+}
+
+#' @export
+fetch_data.macroACS <- function(acs, geography, dataset= c("estimate", "st.err"), 
+                                choice= NULL) {
   dataset <- match.arg(dataset, several.ok= FALSE)
-  choice <- match.arg(choice, several.ok= FALSE)
+  choice <- match.arg(choice, choices= names(acs[["estimates"]]), several.ok= FALSE)
   
-  if (dataset == "estimate") { return(acs$estimates[[choice]]) } 
-  else if (dataset == "st.err") { return(acs$standard_error[[choice]]) }
-  else { stop("input 'dataset' is not valid.") }
-}
-
-#### now the methods ####
-
-#' @title Get Population Data by Age and Sex for Specified Geography
-#' @description Gets data, either estimate or standard error, on a specified geography's population 
-#' by age and sex.
-#' @param acs An object of class \code{"macroACS"}.
-#' @param geography A character vector allowing string matching via \code{\link[base]{grep}} to 
-#' a set of specified geographies. All values may be specified by \code{"*"}.
-#' @param dataset Either \code{"estimate"} or \code{"st.err"}. Do you want data on estimated 
-#' population counts or estimated standard errors?
-#' @export
-get_age_by_sex <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  UseMethod("get_age_by_sex", acs)
-}
-
-#' @export
-get_age_by_sex.macroACS <- function(acs, geography, dataset= c("estimate", "st.err")) {
   ## check inputs
   validate_get_inputs(acs, geography, dataset)
   ## execute return
   if (geography != "*") {
     rowid <- get_rowmatch(geography, acs$geography$NAME)
-    return(fetch_data(acs, dataset, choice= "age_by_sex")[rowid,])
+    if (dataset == "estimate") { return(acs$estimates[[choice]][rowid, ]) } 
+    else if (dataset == "st.err") { return(acs$standard_error[[choice]][rowid, ]) }
+    else { stop("input 'dataset' is not valid.") }
   } else {
-    return(fetch_data(acs, dataset, choice= "age_by_sex"))
+    if (dataset == "estimate") { return(acs$estimates[[choice]]) } 
+    else if (dataset == "st.err") { return(acs$standard_error[[choice]]) }
+    else { stop("input 'dataset' is not valid.") }
   }
 }
 
+##---------------------------------------------------------
+## generics for getting the span, endyear and geography -- macroACS
+##---------------------------------------------------------
+# @title Constructor function for the "macroACS" class
+# @description Constructor function for the "macroACS" class
+# @param endyear An integer specifying the eendyear
+# @param span An integer specifying the span of the data collection period. One of \code{c(1,3,5)}
+# @param estimates a \code{list} of \code{data.frame}s
+# @param standard_error a \code{list} of \code{data.frame}s
+# @param geography A \code{data.frame} specifying the geography to which the data corrresponds
+# @param geo_title An object of class 'geo'.
+new_macroACS <- function(endyear, span, estimates, standard_error, geography,
+                         geo_title) {
+  # validate inputs
+  if (! span %in% c(1,3,5)) stop("The ACS API only supports data spans of 1, 3, and 5 years.")
+  if (endyear %% 1 != 0 | endyear < 2009) stop("endyear must be an integer >= 2009 (when ACS data begins).")
+  if (!is.data.frame(geography)) stop("geography must be a data.frame")
+  #if (!acs::is.geo(geo_title[[1]])) stop("geo_title must be a 'geo' object.")
+  if (!is.list(estimates) & all(!unlist(lapply(estimates, is.data.frame))))
+    stop("estimates must be a list of data.frames.")
+  if (!is.list(standard_error) & all(!unlist(lapply(standard_error, is.data.frame))))
+    stop("standard_error must be a list of data.frames.")
+  
+  # create new macroACS object
+  new_macro <- list(endyear= endyear, span= span,
+                    estimates= estimates,
+                    standard_error= standard_error,
+                    geography= geography,
+                    geo_title= geo_title)
+  class(new_macro) <- "macroACS"
+  return(new_macro)
+}
 
-#' @title Get Population Data by Race for Specified Geography
-#' @description Gets data, either estimate or standard error, on a specified geography's population 
-#' by race.
+
+#' @title Get the span from a "macroACS" object.
+#' @description Get the data collection span from a "macroACS" object
 #' @param acs An object of class \code{"macroACS"}.
-#' @param geography A character vector allowing string matching via \code{\link[base]{grep}} to 
-#' a set of specified geographies. All values may be specified by \code{"*"}.
-#' @param dataset Either \code{"estimate"} or \code{"st.err"}. Do you want data on estimated 
-#' population counts or estimated standard errors?
 #' @export
-get_pop_by_race <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  UseMethod("get_pop_by_race", acs)
+get_span <- function(acs) {
+  UseMethod("get_span", acs)
 }
 
 #' @export
-get_pop_by_race.macroACS <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  ## check inputs
-  validate_get_inputs(acs, geography, dataset)
-  ## execute return
-  if (geography != "*") {
-    rowid <- get_rowmatch(geography, acs$geography$NAME)
-    return(fetch_data(acs, dataset, choice= "pop_by_race")[rowid,])
-  } else {
-    return(fetch_data(acs, dataset, choice= "pop_by_race"))
-  }
+get_span.macroACS <- function(acs) {
+  return(acs$span)
 }
 
-#' @title Get Marital Status for Specified Geography
-#' @description Gets data, either estimate or standard error, on a specified geography's population 
-#' marital status by gender and age.
+#' @title Get the endyear from a "macroACS" object.
+#' @description Get the data collection endyear from a "macroACS" object
 #' @param acs An object of class \code{"macroACS"}.
-#' @param geography A character vector allowing string matching via \code{\link[base]{grep}} to 
-#' a set of specified geographies. All values may be specified by \code{"*"}.
-#' @param dataset Either \code{"estimate"} or \code{"st.err"}. Do you want data on estimated 
-#' population counts or estimated standard errors?
 #' @export
-get_marital_status <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  UseMethod("get_marital_status", acs)
+get_endyear <- function(acs) {
+  UseMethod("get_endyear", acs)
 }
 
 #' @export
-get_marital_status.macroACS <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  ## check inputs
-  validate_get_inputs(acs, geography, dataset)
-  ## execute return
-  if (geography != "*") {
-    rowid <- get_rowmatch(geography, acs$geography$NAME)
-    return(fetch_data(acs, dataset, choice= "marital_status")[rowid,])
-  } else {
-    return(fetch_data(acs, dataset, choice= "marital_status"))
-  }
+get_endyear.macroACS <- function(acs) {
+  return(acs$endyear)
 }
 
-#' @title Get Educational Attainment for Specified Geography
-#' @description Gets data, either estimate or standard error, on a specified geography's population 
-#' educational attainment by gender and age.
+#' @title Get the geography title from a "macroACS" object.
+#' @description Get the summary information of the geography selected from a "macroACS" object
 #' @param acs An object of class \code{"macroACS"}.
-#' @param geography A character vector allowing string matching via \code{\link[base]{grep}} to 
-#' a set of specified geographies. All values may be specified by \code{"*"}.
-#' @param dataset Either \code{"estimate"} or \code{"st.err"}. Do you want data on estimated 
-#' population counts or estimated standard errors?
 #' @export
-get_education <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  UseMethod("get_education", acs)
+get_geography <- function(acs) {
+  UseMethod("get_geography", acs)
 }
 
 #' @export
-get_education.macroACS <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  ## check inputs
-  validate_get_inputs(acs, geography, dataset)
-  ## execute return
-  if (geography != "*") {
-    rowid <- get_rowmatch(geography, acs$geography$NAME)
-    return(fetch_data(acs, dataset, choice= "edu")[rowid,])
-  } else {
-    return(fetch_data(acs, dataset, choice= "edu"))
-  }
+get_geography.macroACS <- function(acs) {
+  return(acs$geo_title)
 }
 
-#' @title Get Nativity Status for Specified Geography
-#' @description Gets data, either estimate or standard error, on a specified geography's population 
-#' nativity status by age.
+#' @title Get dataset names from a "macroACS" object.
+#' @description Get the names of the datasets in a given "macroACS" object. 
 #' @param acs An object of class \code{"macroACS"}.
-#' @param geography A character vector allowing string matching via \code{\link[base]{grep}} to 
-#' a set of specified geographies. All values may be specified by \code{"*"}.
-#' @param dataset Either \code{"estimate"} or \code{"st.err"}. Do you want data on estimated 
-#' population counts or estimated standard errors?
+#' @seealso \code{\link{fetch_data}}
 #' @export
-get_nativity <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  UseMethod("get_nativity", acs)
+get_dataset_names <- function(acs) {
+  UseMethod("get_dataset_names", acs)
 }
 
 #' @export
-get_nativity.macroACS <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  ## check inputs
-  validate_get_inputs(acs, geography, dataset)
-  ## execute return
-  if (geography != "*") {
-    rowid <- get_rowmatch(geography, acs$geography$NAME)
-    return(fetch_data(acs, dataset, choice= "nativity")[rowid,])
-  } else {
-    return(fetch_data(acs, dataset, choice= "nativity"))
-  }
+get_dataset_names.macroACS <- function(acs) {
+  return(names(acs$estimates))
 }
 
-#' @title Get Nativity Status by Income for Specified Geography
-#' @description Gets data, either estimate or standard error, on a specified geography's population 
-#' nativity status by age and income in the previous 12 months.
+#' @title Split a "macroACS" object
+#' @description Split a "macroACS" object into subsets. This may be helpful for users who have
+#' limited memory available on their machines before proceding to derive sample synthetic micro data.
 #' @param acs An object of class \code{"macroACS"}.
-#' @param geography A character vector allowing string matching via \code{\link[base]{grep}} to 
-#' a set of specified geographies. All values may be specified by \code{"*"}.
-#' @param dataset Either \code{"estimate"} or \code{"st.err"}. Do you want data on estimated 
-#' population counts or estimated standard errors?
+#' @param n_splits An integer for the number of splits you wish to create.
+#' @seealso \code{\link{derive_synth_datasets}}
 #' @export
-get_nativity_by_income <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  UseMethod("get_nativity_by_income", acs)
+split <- function(acs, n_splits) {
+  UseMethod("split", acs)
 }
 
 #' @export
-get_nativity_by_income.macroACS <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  ## check inputs
-  validate_get_inputs(acs, geography, dataset)
-  ## execute return
-  if (geography != "*") {
-    rowid <- get_rowmatch(geography, acs$geography$NAME)
-    return(fetch_data(acs, dataset, choice= "by_inc_12mo")[rowid,])
-  } else {
-    return(fetch_data(acs, dataset, choice= "by_inc_12mo"))
+split.macroACS <- function(acs, n_splits) {
+  # keep meta data
+  sp <- get_span(acs)
+  ey <- get_endyear(acs)
+  orig_geo <- acs$geography
+  geo_title <- NULL
+  
+  # split
+  nx <- nrow(acs$geography)
+  split_idx <- parallel::splitIndices(nx, ncl= n_splits)
+  
+  split_macroACS <- vector("list", length= n_splits)
+  for (i in 1:n_splits) {
+    geo <- orig_geo[ split_idx[[i]], ]
+    est <- lapply(acs$estimates, function(l, idx) {return(l[idx, ])}, idx= split_idx[[i]])
+    se <- lapply(acs$standard_error, function(l, idx) {return(l[idx, ])}, idx= split_idx[[i]])
+    
+    split_macroACS[[i]] <- new_macroACS(endyear= ey, span= sp, 
+                                        estimates= est, standard_error= se,
+                                        geography= geo,
+                                        geo_title= geo_title)
   }
+  return(split_macroACS)
 }
 
-#' @title Get Geographic Mobility by Educational Attainment for Specified Geography
-#' @description Gets data, either estimate or standard error, on a specified geography's population 
-#' geographic mobility by educational attainment.
-#' @param acs An object of class \code{"macroACS"}.
-#' @param geography A character vector allowing string matching via \code{\link[base]{grep}} to 
-#' a set of specified geographies. All values may be specified by \code{"*"}.
-#' @param dataset Either \code{"estimate"} or \code{"st.err"}. Do you want data on estimated 
-#' population counts or estimated standard errors?
-#' @export
-get_geographic_mobility <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  UseMethod("get_geographic_mobility", acs)
-}
-
-#' @export
-get_geographic_mobility.macroACS <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  ## check inputs
-  validate_get_inputs(acs, geography, dataset)
-  ## execute return
-  if (geography != "*") {
-    rowid <- get_rowmatch(geography, acs$geography$NAME)
-    return(fetch_data(acs, dataset, choice= "geo_mob_edu")[rowid,])
-  } else {
-    return(fetch_data(acs, dataset, choice= "geo_mob_edu"))
-  }
-}
-
-#' @title Get Individual Income for Specified Geography
-#' @description Gets data, either estimate or standard error, on a specified geography's population 
-#' income in the past 12 months.
-#' @param acs An object of class \code{"macroACS"}.
-#' @param geography A character vector allowing string matching via \code{\link[base]{grep}} to 
-#' a set of specified geographies. All values may be specified by \code{"*"}.
-#' @param dataset Either \code{"estimate"} or \code{"st.err"}. Do you want data on estimated 
-#' population counts or estimated standard errors?
-#' @export
-get_ind_income <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  UseMethod("get_ind_income", acs)
-}
-
-#' @export
-get_ind_income.macroACS <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  ## check inputs
-  validate_get_inputs(acs, geography, dataset)
-  ## execute return
-  if (geography != "*") {
-    rowid <- get_rowmatch(geography, acs$geography$NAME)
-    return(fetch_data(acs, dataset, choice= "ind_inc")[rowid,])
-  } else {
-    return(fetch_data(acs, dataset, choice= "ind_inc"))
-  }
-}
-
-#' @title Get Employment Status for Specified Geography
-#' @description Gets data, either estimate or standard error, on a specified geography's population 
-#' employment status in the prior 12 months by gender and age.
-#' @param acs An object of class \code{"macroACS"}.
-#' @param geography A character vector allowing string matching via \code{\link[base]{grep}} to 
-#' a set of specified geographies. All values may be specified by \code{"*"}.
-#' @param dataset Either \code{"estimate"} or \code{"st.err"}. Do you want data on estimated 
-#' population counts or estimated standard errors?
-#' @export
-get_employment_status <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  UseMethod("get_employment_status", acs)
-}
-
-#' @export
-get_employment_status.macroACS <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  ## check inputs
-  validate_get_inputs(acs, geography, dataset)
-  ## execute return
-  if (geography != "*") {
-    rowid <- get_rowmatch(geography, acs$geography$NAME)
-    return(fetch_data(acs, dataset, choice= "emp_status")[rowid,])
-  } else {
-    return(fetch_data(acs, dataset, choice= "emp_status"))
-  }
-}
-
-#' @title Get Poverty Status for Specified Geography
-#' @description Gets data, either estimate or standard error, on a specified geography's population 
-#' poverty status in the prior 12 months by gender and employment status.
-#' @param acs An object of class \code{"macroACS"}.
-#' @param geography A character vector allowing string matching via \code{\link[base]{grep}} to 
-#' a set of specified geographies. All values may be specified by \code{"*"}.
-#' @param dataset Either \code{"estimate"} or \code{"st.err"}. Do you want data on estimated 
-#' population counts or estimated standard errors?
-#' @export
-get_poverty_status <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  UseMethod("get_poverty_status", acs)
-}
-
-#' @export
-get_poverty_status.macroACS <- function(acs, geography, dataset= c("estimate", "st.err")) {
-  ## check inputs
-  validate_get_inputs(acs, geography, dataset)
-  ## execute return
-  if (geography != "*") {
-    rowid <- get_rowmatch(geography, acs$geography$NAME)
-    return(fetch_data(acs, dataset, choice= "pov_status1")[rowid,])
-  } else {
-    return(fetch_data(acs, dataset, choice= "pov_status1"))
-  }
-}
 
 ##---------------------------------------------------------
 ## GENERICS FOR CLASS "macro_micro" -- SPECIFICALLY ADDING CONSTRAINT LISTS
@@ -899,3 +795,46 @@ all_geog_constraint_race.synthACS <- function(obj, method= c("synthetic", "macro
   }
   return(constraint)
 }
+
+
+##---------------------------------------------------------
+## Generics for class smsm_set
+##---------------------------------------------------------
+#' @title Summarizing SMSM fits
+#' @description \code{summary} method for class 'smsm_set'. 
+#' @param object An object of class \code{'smsm_set'}, typically a result of call to 
+#' \code{\link{all_geog_optimize_microdata}}
+#' @param ... additional arguments affecting the summary produced.
+#' @export
+summary.smsm_set <- function(object, ...) {
+  
+  tae_pct <- unlist(object$tae) / unlist(lapply(object$best_fit, nrow)) / object$D
+  tae_q <- round(stats::quantile(tae_pct, c(0,.25,.5,.75,.9,.95)), 6)
+  names(tae_q) <- c("0%", "25%", "50%", "75%", "90%", "95%")
+  n_early <- sum(unlist(object$iter) < object$max_iter)
+  
+  cat("\n Call: \n", paste(deparse(object$call), collapse= "\n"),
+      "\n \n Seed: ", object$seed,
+      "\n n-Constraints: ", object$D,
+      "\n \n Maximum Iterations: ", object$max_iter,
+      "\n %-Early Stop: ", round(n_early / length(object$iter), 4),
+      "\n \n Mean %-TAE: ", round(mean(tae_pct), 6),
+      "\n Median %-TAE: ", round(stats::median(tae_pct), 6),
+      "\n Max %-TAE: ", round(max(tae_pct), 6),
+      "\n %-TAE quantiles: \n ")
+  print(tae_q)
+}
+
+
+#' #' @title Combine separate SMSM optimizations
+#' #' @description Combine multiple objects of class "smsm_set" into a single object of class "smsm_set"
+#' #' @param ... objects of class 'smsm_set'.
+#' #' @seealso \code{\link[synthACS]{split}}
+#' #' @export
+#' combine_smsm <- function(...) {
+#'   UseMethod("combine_smsm", ...)
+#' }
+
+
+## combine, Z-statistics???
+
